/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import {
  Direction,
  ON_PAGE,
  PostfachMailLinkRel,
  PostfachMailResource,
} from '@alfa-client/postfach-shared';
import { HasLinkPipe, StateResource, createStateResource } from '@alfa-client/tech-shared';
import {
  existsAsHtmlElement,
  getMockComponent,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { IncommingMailComponent } from './incomming-mail/incomming-mail.component';
import { OutgoingMailComponent } from './outgoing-mail/outgoing-mail.component';
import { PostfachMailAttachmentsComponent } from './postfach-mail-attachments/postfach-mail-attachments.component';
import { PostfachMailComponent } from './postfach-mail.component';

describe('PostfachMailComponent', () => {
  let component: PostfachMailComponent;
  let fixture: ComponentFixture<PostfachMailComponent>;

  const postfachIncomingNachricht: string = getDataTestIdOf('postfach-incoming-nachricht');
  const postfachOutgoingNachricht: string = getDataTestIdOf('postfach-outgoing-nachricht');
  const postfachNachrichtAttachments: string = getDataTestIdOf(
    'postfach-mail-attachments-container',
  );

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        HasLinkPipe,
        PostfachMailComponent,
        MockComponent(IncommingMailComponent),
        MockComponent(OutgoingMailComponent),
        MockComponent(PostfachMailAttachmentsComponent),
      ],
      providers: [
        {
          provide: ON_PAGE,
          useValue: undefined,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PostfachMailComponent);
    component = fixture.componentInstance;
    component.postfachMail = createPostfachMailResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('attachments component', () => {
    it('should not be visible on missing link', () => {
      component.onPage = true;
      component.postfachMail = createPostfachMailResource();

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, postfachNachrichtAttachments);
    });

    describe('on existing link and page', () => {
      const postfachNachricht: PostfachMailResource = createPostfachMailResource([
        PostfachMailLinkRel.ATTACHMENTS,
      ]);

      beforeEach(() => {
        component.onPage = true;
        component.postfachMail = postfachNachricht;
      });

      it('should be visible', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, postfachNachrichtAttachments);
      });

      it('should be called with postfachNachricht', () => {
        fixture.detectChanges();

        const postfachAttachmentsComponent: PostfachMailAttachmentsComponent = getMockComponent(
          fixture,
          PostfachMailAttachmentsComponent,
        );

        expect(postfachAttachmentsComponent.postfachNachricht).toBe(postfachNachricht);
      });
    });
  });

  describe('outgoing nachricht component', () => {
    describe('on outgoing nachricht', () => {
      const outgoingPostfachNachricht: PostfachMailResource = {
        ...createPostfachMailResource(),
        direction: Direction.OUT,
      };
      const vorgangStateResource: StateResource<VorgangWithEingangResource> = createStateResource(
        createVorgangWithEingangResource(),
      );

      beforeEach(() => {
        component.postfachMail = outgoingPostfachNachricht;
        component.vorgangStateResource = vorgangStateResource;
        fixture.detectChanges();
      });

      describe('call', () => {
        it('should be with postfachNachricht', () => {
          const outgoingMailComponent: OutgoingMailComponent = getMockComponent(
            fixture,
            OutgoingMailComponent,
          );

          expect(outgoingMailComponent.postfachMail).toBe(outgoingPostfachNachricht);
        });

        it('should be with vorgangStateResource', () => {
          const outgoingMailComponent: OutgoingMailComponent = getMockComponent(
            fixture,
            OutgoingMailComponent,
          );

          expect(outgoingMailComponent.vorgangStateResource).toBe(vorgangStateResource);
        });
      });

      it('should be visible', () => {
        existsAsHtmlElement(fixture, postfachOutgoingNachricht);
      });
    });

    describe('on incoming nachricht', () => {
      const incomingPostfachNachricht: PostfachMailResource = {
        ...createPostfachMailResource(),
        direction: Direction.IN,
      };

      beforeEach(() => {
        component.postfachMail = incomingPostfachNachricht;
        fixture.detectChanges();
      });

      it('should not be visible', () => {
        notExistsAsHtmlElement(fixture, postfachOutgoingNachricht);
      });
    });
  });

  describe('incoming nachricht component', () => {
    describe('on incoming nachricht', () => {
      const incomingPostfachNachricht: PostfachMailResource = {
        ...createPostfachMailResource(),
        direction: Direction.IN,
      };

      beforeEach(() => {
        component.postfachMail = incomingPostfachNachricht;
        fixture.detectChanges();
      });

      describe('call', () => {
        it('should be with postfachNachricht', () => {
          const incomingMailComponent: IncommingMailComponent = getMockComponent(
            fixture,
            IncommingMailComponent,
          );

          expect(incomingMailComponent.postfachMail).toBe(incomingPostfachNachricht);
        });
      });

      it('should be visible', () => {
        existsAsHtmlElement(fixture, postfachIncomingNachricht);
      });
    });

    describe('on outgoing nachricht', () => {
      const outgoingPostfachNachricht: PostfachMailResource = {
        ...createPostfachMailResource(),
        direction: Direction.OUT,
      };

      beforeEach(() => {
        component.postfachMail = outgoingPostfachNachricht;
        fixture.detectChanges();
      });

      it('should not be visible', () => {
        notExistsAsHtmlElement(fixture, postfachIncomingNachricht);
      });
    });
  });
});
