/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  Bescheid,
  BescheidLinkRel,
  BescheidResource,
  BescheidService,
  DocumentResource,
  UploadFileInProgress,
} from '@alfa-client/bescheid-shared';
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import {
  CommandOrder,
  CommandResource,
  switchMapCommandSuccessfullyDone,
  tapOnCommandSuccessfullyDone,
} from '@alfa-client/command-shared';
import { createEmptyStateResource, isLoaded, StateResource } from '@alfa-client/tech-shared';
import { Component, EventEmitter, OnInit, Output } from '@angular/core';
import { hasLink, Resource } from '@ngxp/rest';
import { BehaviorSubject, filter, map, Observable, of, OperatorFunction, tap } from 'rxjs';
import { BescheidenFormService } from '../bescheiden.formservice';

type sendBescheid = (
  BescheidResource: BescheidResource,
) => Observable<StateResource<CommandResource>>;

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-result',
  templateUrl: './vorgang-detail-bescheiden-result.component.html',
})
export class VorgangDetailBescheidenResultComponent implements OnInit {
  static readonly ADD_DOCUMENTS_STEP: number = 2;
  static readonly BESCHEID_VERSENDEN_STEP: number = 3;

  @Output() closeDialog: EventEmitter<void> = new EventEmitter();

  public activeStep$: Observable<number>;
  public bescheid$: Observable<Bescheid>;
  public sendByManual$: Observable<boolean>;

  public bescheidDraftStateResource$: Observable<StateResource<BescheidResource>> = of(
    createEmptyStateResource<BescheidResource>(),
  );
  public bescheidDocumentFile$: Observable<StateResource<BinaryFileResource>> = of(
    createEmptyStateResource<BinaryFileResource>(),
  );
  public bescheidDocument$: Observable<StateResource<DocumentResource>> = of(
    createEmptyStateResource<DocumentResource>(),
  );
  public saveAndSendInProgress$: Observable<StateResource<CommandResource>> = of(
    createEmptyStateResource<CommandResource>(),
  );

  public uploadBescheidDocumentInProgress$: Observable<UploadFileInProgress> = of({
    loading: false,
  });

  public createBescheidDocumentInProgress$: Observable<StateResource<CommandResource>> = of(
    createEmptyStateResource<CommandResource>(),
  );

  public sendWithNachricht$: BehaviorSubject<StateResource<CommandResource>> = new BehaviorSubject(
    createEmptyStateResource<CommandResource>(),
  );

  public canSave$: Observable<boolean> = of(true);
  public canSend$: Observable<boolean> = of(true);
  public showMissingBescheidDocumentError$: Observable<boolean> = of(false);

  public bescheidLinkRel = BescheidLinkRel;

  constructor(
    private bescheidService: BescheidService,
    public formService: BescheidenFormService,
  ) {}

  ngOnInit(): void {
    this.bescheidDraftStateResource$ = this.bescheidService.getBescheidDraft();
    this.bescheidDocumentFile$ = this.bescheidService.getBescheidDocumentFile();
    this.uploadBescheidDocumentInProgress$ =
      this.bescheidService.getUploadBescheidDocumentInProgress();
    this.createBescheidDocumentInProgress$ =
      this.bescheidService.getCreateBescheidDocumentInProgress();
    this.bescheidDocument$ = this.bescheidService.getBescheidDocument();

    this.activeStep$ = this.getActiveStep();
    this.bescheid$ = this.formService.getBescheidChanges();
    this.sendByManual$ = this.getIsSendManually();

    this.canSave$ = this.bescheidDraftStateResource$.pipe(
      filter(isLoaded),
      map((stateResource: StateResource<BescheidResource>) =>
        hasLink(stateResource.resource, BescheidLinkRel.BESCHEIDEN),
      ),
    );
    this.canSend$ = this.bescheidDraftStateResource$.pipe(
      filter(isLoaded),
      map((stateResource: StateResource<BescheidResource>) =>
        hasLink(stateResource.resource, BescheidLinkRel.BESCHEIDEN_UND_SENDEN),
      ),
    );
    this.showMissingBescheidDocumentError$ = this.formService.getShowMissingBescheidDocumentError();
  }

  getActiveStep(): Observable<number> {
    return this.formService
      .getActiveStep()
      .pipe(tap((step: number) => this.resetStateOnStepChange(step)));
  }

  getIsSendManually(): Observable<boolean> {
    return this.formService.isSendByManual().pipe(tap(() => this.resetSend()));
  }

  private resetStateOnStepChange(step: number): void {
    if (this.isNotLast(step)) this.resetSend();
    if (this.shouldClearAttachmentInProgress(step)) this.bescheidService.clearAttachmentUpload();
  }

  private isNotLast(step: number): boolean {
    return step < VorgangDetailBescheidenResultComponent.BESCHEID_VERSENDEN_STEP;
  }

  private shouldClearAttachmentInProgress(step: number): boolean {
    return step != VorgangDetailBescheidenResultComponent.ADD_DOCUMENTS_STEP;
  }

  resetSend(): void {
    this.saveAndSendInProgress$ = of(createEmptyStateResource<CommandResource>());
    this.sendWithNachricht$.next(createEmptyStateResource<CommandResource>());
  }

  public deleteBescheidDocument(): void {
    this.formService.clearBescheidDocumentFile();
    this.bescheidService.deleteBescheidDocument();
  }

  public saveAndSendManually(bescheidDraft: BescheidResource): void {
    this.formService.clearNachricht();
    this.saveAndSendInProgress$ = this.doUpdateAndSend(bescheidDraft, (bescheidResource) =>
      this.bescheidService.sendBescheidManually(bescheidResource),
    );
  }

  public saveAndSendWithNachricht(bescheidDraft: BescheidResource): void {
    this.saveAndSendInProgress$ = this.doUpdateAndSend(bescheidDraft, (bescheidResource) =>
      this.bescheidService
        .sendBescheidToAntragsteller(bescheidResource)
        .pipe(
          tap((sendCommandStateResource: StateResource<CommandResource>) =>
            this.sendWithNachricht$.next(sendCommandStateResource),
          ),
        ),
    );
  }

  doUpdateAndSend(
    bescheidDraft: BescheidResource,
    send: sendBescheid,
  ): Observable<StateResource<CommandResource>> {
    return this.formService.submit(this.doSendAfterSuccessfullyUpdate(bescheidDraft, send)).pipe(
      tapOnCommandSuccessfullyDone((commandStateResource: StateResource<CommandResource>) => {
        if (this.isSendBescheidCommand(commandStateResource)) {
          this.closeDialog.emit();
        }
      }),
    );
  }

  doSendAfterSuccessfullyUpdate(
    bescheidDraft: BescheidResource,
    send: sendBescheid,
  ): OperatorFunction<StateResource<Resource>, StateResource<Resource>> {
    return switchMapCommandSuccessfullyDone(() => send(bescheidDraft));
  }

  private isSendBescheidCommand(commandStateResource: StateResource<CommandResource>): boolean {
    return commandStateResource.resource.order == CommandOrder.SEND_BESCHEID;
  }
}
