/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.function.Predicate;
import java.util.stream.Stream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.bescheid.BescheidController;
import de.ozgcloud.alfa.bescheid.DocumentController;
import de.ozgcloud.alfa.collaboration.CollaborationController.CollaborationByVorgangController;
import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.kommentar.KommentarController;
import de.ozgcloud.alfa.postfach.PostfachMailController;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.forwarding.ForwardingController;
import de.ozgcloud.alfa.wiedervorlage.WiedervorlageController;

@Component
class CommandModelAssembler implements RepresentationModelAssembler<Command, EntityModel<Command>> {

	static final LinkRelation REL_EFFECTED_RESOURCE = LinkRelation.of("effected_resource");
	static final LinkRelation REL_REVOKE = LinkRelation.of("revoke");
	static final LinkRelation REL_UPDATE = LinkRelation.of("update");

	static final Predicate<Command> IS_NOT_LOESCH_ANFORDERUNG_AND_REVOKEABLE = command -> !CommandHelper.IS_LOESCHANFORDERUNG.test(command)
			&& CommandHelper.IS_REVOKEABLE.test(command);

	static final Predicate<Command> HAS_KNOWN_COMMAND_ORDER = command -> command.getCommandOrder() != CommandOrder.UNBEKANNT;

	@Override
	public EntityModel<Command> toModel(Command command) {
		return ModelBuilder.fromEntity(command)
				.addLink(linkTo(CommandController.class).slash(command.getId()).withSelfRel())
				.ifMatch(CommandHelper.IS_DONE.and(HAS_KNOWN_COMMAND_ORDER))
				.addLink(() -> effectedResourceLinkByOrderType(command))
				.ifMatch(CommandHelper.IS_PENDING)
				.addLink(() -> linkTo(CommandController.class).slash(command.getId()).withRel(REL_UPDATE))
				.ifMatch(IS_NOT_LOESCH_ANFORDERUNG_AND_REVOKEABLE)
				.addLink(() -> linkTo(methodOn(CommandController.class).revoke(command.getId(), null)).withRel(REL_REVOKE))
				.buildModel();
	}

	Link effectedResourceLinkByOrderType(Command entity) {
		var type = entity.getCommandOrder().getType();

		WebMvcLinkBuilder linkBuilder = switch (type) {
			case FORWARDING -> linkTo(methodOn(ForwardingController.class).findByVorgangId(entity.getVorgangId()));
			case KOMMENTAR -> linkTo(KommentarController.class).slash(entity.getRelationId());
			case POSTFACH -> linkTo(methodOn(PostfachMailController.class).getAll(entity.getVorgangId()));
			case VORGANG -> linkTo(VorgangController.class).slash(entity.getRelationId());
			case VORGANG_LIST -> linkTo(VorgangController.class);
			case WIEDERVORLAGE -> linkTo(WiedervorlageController.class).slash(entity.getRelationId());
			case BESCHEID -> linkTo(methodOn(BescheidController.class).getDraft(entity.getVorgangId()));
			case DOCUMENT -> linkTo(DocumentController.class).slash(entity.getCreatedResource());
			case COLLABORATION -> linkTo(methodOn(CollaborationByVorgangController.class).getAllByVorgangId(entity.getVorgangId()));
			case NONE -> throw new IllegalArgumentException("Unknown CommandOrder: " + entity.getOrder());
		};

		return linkBuilder.withRel(REL_EFFECTED_RESOURCE);
	}

	public CollectionModel<EntityModel<Command>> toCollectionModel(Stream<Command> entities) {
		return CollectionModel.of(entities.map(this::toModel).toList(), linkTo(CommandController.class).withSelfRel());
	}
}