/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.alfa.vorgang.forwarding.RedirectRequest;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.grpc.command.CommandServiceGrpc.CommandServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.command.GrpcCreateCommandRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcExistsPendingCommandsRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcFindCommandsRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcGetCommandRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcGetPendingCommandsRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcRedirectRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcRevokeCommandRequest;
import de.ozgcloud.vorgang.grpc.command.GrpcSetCommandExecutedRequest;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
public class CommandRemoteService {
	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private CommandServiceBlockingStub commandServiceStub;
	@Autowired
	private CommandMapper mapper;
	@Autowired
	private CommandBodyMapper bodyMapper;

	@Autowired
	private ContextService contextService;

	@Autowired
	private GrpcObjectMapper objectMapper;

	public Command createCommand(CreateCommand command) {
		var response = commandServiceStub.createCommand(buildCreateCommandRequest(command));

		return mapper.toCommand(response.getCommand());
	}

	GrpcCreateCommandRequest buildCreateCommandRequest(CreateCommand command) {
		var requestBuilder = GrpcCreateCommandRequest.newBuilder()
				.setCallContext(contextService.createCallContext())
				.setVorgangId(command.getVorgangId())
				.setRelationId(command.getRelationId())
				.setRelationVersion(command.getRelationVersion())
				.setOrderString(command.getOrder())
				.addAllBody(bodyMapper.mapToBodyFields(command.getBody()))
				.setBodyObj(objectMapper.fromMap(bodyMapper.fromObjectToMap(command.getBody())));

		Optional.ofNullable(command.getRedirectRequest()).map(this::buildForwardRequest).ifPresent(requestBuilder::setRedirectRequest);

		return requestBuilder.build();
	}

	GrpcRedirectRequest buildForwardRequest(RedirectRequest forwardRequest) {
		var builder = GrpcRedirectRequest.newBuilder().setEmail(forwardRequest.getEmail());
		if (ArrayUtils.isNotEmpty(forwardRequest.getPassword())) {
			builder.setPassword(String.valueOf(forwardRequest.getPassword()));
		}
		return builder.build();
	}

	public Command revokeCommand(String commandId) {
		var response = commandServiceStub.revokeCommand(createRevokeRequest(commandId));

		return mapper.toCommand(response.getCommand());
	}

	GrpcRevokeCommandRequest createRevokeRequest(String commandId) {
		return GrpcRevokeCommandRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setId(commandId)
				.build();
	}

	public Command getCommand(String commandId) {
		var response = commandServiceStub.getCommand(createGetCommandRequest(commandId));

		return mapper.toCommand(response);
	}

	GrpcGetCommandRequest createGetCommandRequest(String commandId) {
		return GrpcGetCommandRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setId(commandId)
				.build();
	}

	public boolean existsPendingCommands(String relationId) {
		var response = commandServiceStub.existsPendingCommands(buildGrpcExistsPendingCommandsRequest(relationId));

		return response.getExistsPendingCommands();
	}

	private GrpcExistsPendingCommandsRequest buildGrpcExistsPendingCommandsRequest(String relationId) {
		return GrpcExistsPendingCommandsRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setVorgangId(relationId)
				.build();
	}

	public Stream<Command> getPendingCommands(String vorgangId) {
		var response = commandServiceStub.getPendingCommands(buildGrpcGetPendingCommandsRequest(vorgangId));

		return response.getCommandList().stream().map(mapper::toCommand);
	}

	private GrpcGetPendingCommandsRequest buildGrpcGetPendingCommandsRequest(String vorgangId) {
		return GrpcGetPendingCommandsRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setVorgangId(vorgangId)
				.build();
	}

	public Stream<Command> findCommands(String vorgangId, Optional<CommandStatus> status, Optional<String> order) {
		var response = commandServiceStub.findCommands(buildFindCommandsRequest(vorgangId, status, order));

		return response.getCommandList().stream().map(mapper::toCommand);
	}

	private GrpcFindCommandsRequest buildFindCommandsRequest(String vorgangId, Optional<CommandStatus> status, Optional<String> order) {
		var builder = GrpcFindCommandsRequest.newBuilder()
				.setContext(contextService.createCallContext())
				.setVorgangId(vorgangId);

		status.map(CommandStatus::name).ifPresent(builder::addStatus);
		order.ifPresent(builder::setOrderString);

		return builder.build();
	}

	public void finishCommand(String commandId) {
		commandServiceStub.setCommandExecuted(buildGrpcSetCommandExecutedRequest(commandId));
	}

	private GrpcSetCommandExecutedRequest buildGrpcSetCommandExecutedRequest(String commandId) {
		return GrpcSetCommandExecutedRequest.newBuilder().setCommandId(commandId).build();
	}
}