/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.historie;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.historie.CommandWithPrevious.CommandWithPreviousBuilder;

abstract class ChangeHistoryBuilder<T extends ChangeHistoryBuilder<T>> {

	List<Command> commands;
	String organisationseinheitenID;

	@SuppressWarnings("unchecked")
	public T withCommands(List<Command> commands) {
		this.commands = commands;
		return (T) this;
	}

	@SuppressWarnings("unchecked")
	public T withOrganisationseinheitenID(String organisationseinheitenID) {
		this.organisationseinheitenID = organisationseinheitenID;
		return (T) this;
	}

	public Stream<VorgangChange> build() {
		return Stream.of(retrieveRelevantCommands())
				.map(this::inChronologicalOrder)
				.map(this::addPreviousCommand)
				.map(this::toCommandsWithChangeValues)
				.flatMap(this::toVorgangChanges);
	}

	Stream<Command> retrieveRelevantCommands() {
		return commands.stream().filter(this::isRelevant);
	}

	abstract boolean isRelevant(Command command);

	Stream<Command> inChronologicalOrder(Stream<Command> commands) {
		return commands.sorted(Comparator.comparing(Command::getFinishedAt));
	}

	Stream<CommandWithPrevious> addPreviousCommand(Stream<Command> commands) {
		var commandsList = commands.toList();
		var result = new ArrayList<CommandWithPrevious>(commandsList.size());
		for (int i = 0; i < commandsList.size(); i++) {
			CommandWithPreviousBuilder builder = new CommandWithPreviousBuilder().command(commandsList.get(i));
			if (i > 0) {
				builder.previous(commandsList.get(i - 1));
			}
			result.add(builder.build());
		}
		return result.stream();
	}

	Stream<CommandWithChangeValues> toCommandsWithChangeValues(Stream<CommandWithPrevious> commands) {
		return commands.map(this::toCommandWithChangeValues);
	}

	abstract CommandWithChangeValues toCommandWithChangeValues(CommandWithPrevious commandWithPrevious);

	Stream<VorgangChange> toVorgangChanges(Stream<CommandWithChangeValues> commandsWithChangeValues) {
		return commandsWithChangeValues.map(this::toVorgangChange);
	}

	VorgangChange toVorgangChange(CommandWithChangeValues commandChangeValues) {
		return VorgangChange.builder()
				.valueBeforeChange(commandChangeValues.valueBeforeChange())
				.valueAfterChange(commandChangeValues.valueAfterChange())
				.authorFullName(commandChangeValues.command().getCreatedByName())
				.finishedAt(commandChangeValues.command().getFinishedAt())
				.order(commandChangeValues.command().getOrder())
				.organisationseinheitenID(organisationseinheitenID)
				.build();
	}

	String getValueFromCommandBody(String propertyName, Command command) {
		return Optional.ofNullable(command.getBody()).map(body -> body.get(propertyName)).map(Object::toString).orElse(StringUtils.EMPTY);
	}

	record CommandWithChangeValues(Command command, String valueBeforeChange, String valueAfterChange) {
	}
}
