/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.junit.Assert.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Function;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileRequest;
import io.grpc.stub.CallStreamObserver;
import lombok.SneakyThrows;

class ChunkedFileSenderTest {

	private CallStreamObserver<GrpcUploadBinaryFileRequest> requestObserver = mock(CallStreamObserver.class);
	private InputStream uploadStream = mock(InputStream.class);

	private int chunkSize = BinaryFileTestFactory.DATA.length;
	private GrpcUploadBinaryFileRequest uploadBinaryFileRequest = GrpcUploadBinaryFileRequestTestFactory.createDataRequest();
	private Function<byte[], GrpcUploadBinaryFileRequest> buildUploadBinaryFileRequest = when(mock(Function.class).apply(any()))
			.thenReturn(uploadBinaryFileRequest).getMock();
	private GrpcUploadBinaryFileRequest metadataRequest = GrpcUploadBinaryFileRequestTestFactory.createMetadataRequest();

	private ChunkedFileSender<GrpcUploadBinaryFileRequest> streamer = new ChunkedFileSender<>(uploadStream, chunkSize, buildUploadBinaryFileRequest,
			metadataRequest);

	@Nested
	class TestSendBinaryFile {

		@Test
		void shouldNotSendWhenDone() {
			ReflectionTestUtils.setField(streamer, "hasUploadFile", new AtomicBoolean(false));

			streamer.sendChunkTo(requestObserver);

			verify(requestObserver, never()).onNext(any());
		}

		@SneakyThrows
		@Test
		void shouldCloseUploadStream() {
			when(uploadStream.readNBytes(anyInt())).thenReturn(new byte[] {});

			streamer.sendChunkTo(requestObserver);

			verify(uploadStream).close();
		}

		@SneakyThrows
		@Test
		void shouldCallOnCompleted() {
			when(uploadStream.readNBytes(anyInt())).thenReturn(new byte[] {});

			streamer.sendChunkTo(requestObserver);

			verify(requestObserver).onCompleted();
		}

		@SneakyThrows
		@Test
		void shouldCallOnNext() {
			when(uploadStream.readNBytes(anyInt())).thenReturn(BinaryFileTestFactory.DATA);

			streamer.sendChunkTo(requestObserver);

			verify(requestObserver).onNext(uploadBinaryFileRequest);
		}

		@SneakyThrows
		@Test
		void shouldThrowException() {
			doThrow(IOException.class).when(uploadStream).readNBytes(anyInt());

			assertThrows(TechnicalException.class, () -> streamer.sendChunkTo(requestObserver));
		}

		@SneakyThrows
		@Test
		void shouldBuildRequest() {
			when(uploadStream.readNBytes(anyInt())).thenReturn(BinaryFileTestFactory.DATA);

			streamer.sendChunkTo(requestObserver);

			verify(buildUploadBinaryFileRequest).apply(BinaryFileTestFactory.DATA);
		}

		@SneakyThrows
		@Test
		void shouldSendMetadata() {
			when(uploadStream.readNBytes(anyInt())).thenReturn(BinaryFileTestFactory.DATA);

			streamer.sendChunkTo(requestObserver);

			verify(requestObserver).onNext(metadataRequest);
		}
	}
}