/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.historie;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Map;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CommandStatus;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.alfa.loeschanforderung.DeleteLoeschAnforderung;
import de.ozgcloud.alfa.postfach.PostfachMail;
import de.ozgcloud.alfa.wiedervorlage.Wiedervorlage;

class HistorieCommandHandlerTest {

	@Spy
	@InjectMocks
	private HistorieCommandHandler handler;

	@Mock
	private VorgangAttachedItemService vorgangAttachedItemService;

	@Mock
	private CommandService commandService;

	@DisplayName("Is historie command")
	@Nested
	class TestIsHistorieCommand {

		@Test
		void shouldReturnTrueIfNoItemNameExists() {
			var result = handler
					.isHistorieCommand(Command.builder().order(CommandOrder.CREATE_ATTACHED_ITEM.name()).body(Collections.emptyMap()).build());

			assertThat(result).isTrue();
		}

		@Test
		void shouldCallIsLoeschAnforderungZuruecknehmenRevoked() {
			var command = CommandTestFactory.create();

			handler.isHistorieCommand(command);

			verify(handler).isLoeschAnforderungZuruecknehmenRevoked(command);
		}

		@Test
		void shouldReturnFalseOnLoeschAnforderungZuruecknehmenRevoked() {
			doReturn(true).when(handler).isLoeschAnforderungZuruecknehmenRevoked(any());

			var result = handler.isHistorieCommand(CommandTestFactory.create());

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnFalseOnDeleteVorgangAttachedItem() {
			var result = handler.isHistorieCommand(CommandTestFactory.createBuilder().order(CommandOrder.DELETE_ATTACHED_ITEM.name()).build());

			assertThat(result).isFalse();
		}

		@Test
		void shouldCallIsDeleteVorgangAttachedItem() {
			var command = CommandTestFactory.create();

			handler.isHistorieCommand(command);

			verify(handler).isDeleteVorgangAttachedItem(command);
		}
	}

	@DisplayName("postfach nachricht verification")
	@Nested
	class TestIsOutgoingPostfachNachrichtenByOzgCloudNachrichtenManager {

		private final Command matchingCommand = CommandTestFactory.createBuilder()
				.order(CommandOrder.CREATE_ATTACHED_ITEM.name())
				.body(Map.of("item", Map.of(HistorieCommandHandler.DIRECTION, HistorieCommandHandler.DIRECTION_OUTGOING),
						HistorieCommandHandler.CLIENT, HistorieCommandHandler.OZGCLOUD_NACHRICHTEN_MANAGER))
				.build();

		@DisplayName("should return true if the command is CREATED_ATTACHED_ITEM order outgoing by mailserice")
		@Test
		void shouldReturnTrue() {
			var result = handler.isOutgoingPostfachNachrichtByOzgCloudNachrichtenManager(matchingCommand);

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalseOnIncoming() {
			var nonMatchingCommand = matchingCommand.toBuilder()
					.body(Map.of("item", Map.of(HistorieCommandHandler.DIRECTION, HistorieCommandHandler.DIRECTION_INCOMMING),
							HistorieCommandHandler.CLIENT, HistorieCommandHandler.OZGCLOUD_NACHRICHTEN_MANAGER))
					.build();

			var result = handler.isOutgoingPostfachNachrichtByOzgCloudNachrichtenManager(nonMatchingCommand);

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnFalseOnOtherClient() {
			var nonMatchingCommand = matchingCommand.toBuilder()
					.body(Map.of("item", Map.of(HistorieCommandHandler.DIRECTION, HistorieCommandHandler.DIRECTION_OUTGOING),
							HistorieCommandHandler.CLIENT, "Alfa"))
					.build();

			var result = handler.isOutgoingPostfachNachrichtByOzgCloudNachrichtenManager(nonMatchingCommand);

			assertThat(result).isFalse();
		}
	}

	@Nested
	class TestIsLoscheAnforderungVorgangAttachedItem {

		@Test
		void shouldReturnFalse() {
			when(vorgangAttachedItemService.isLoeschAnforderung(VorgangAttachedItemTestFactory.ITEM_NAME)).thenReturn(false);

			var result = handler.isCreateLoeschAnforderungVorgangAttachedItem(Command.builder().order(CommandOrder.CREATE_ATTACHED_ITEM.name())
					.body(Map.of(
							VorgangAttachedItem.FIELD_ITEM_NAME,
							VorgangAttachedItemTestFactory.ITEM_NAME))
					.build());

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnTrue() {
			when(vorgangAttachedItemService.isLoeschAnforderung(VorgangAttachedItemTestFactory.ITEM_NAME)).thenReturn(true);

			var result = handler.isCreateLoeschAnforderungVorgangAttachedItem(Command.builder().order(CommandOrder.CREATE_ATTACHED_ITEM.name())
					.body(Map.of(
							VorgangAttachedItem.FIELD_ITEM_NAME,
							VorgangAttachedItemTestFactory.ITEM_NAME))
					.build());

			assertThat(result).isTrue();
		}
	}

	@DisplayName("Is loesch anforderung zuruecknehmen revoked")
	@Nested
	class TestIsLoeschAnforderungZuruecknehmenRevoked {

		@DisplayName("on matchin order")
		@Nested
		class TestOnMatchingOrder {

			private final Command loeschAnforderungZuruecknehmen = CommandTestFactory.createBuilder()
					.order(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN.name())
					.body(Map.of(DeleteLoeschAnforderung.CHANGE_STATUS_COMMAND_ID_FIELD, CommandTestFactory.ID))
					.build();

			@Test
			void shouldCallCommandService() {
				when(commandService.getById(anyString())).thenReturn(CommandTestFactory.create());

				handler.isLoeschAnforderungZuruecknehmenRevoked(loeschAnforderungZuruecknehmen);

				verify(commandService).getById(CommandTestFactory.ID);
			}

			@Test
			void shouldReturnTrueOnRevokedStatus() {
				var revokedCommand = loeschAnforderungZuruecknehmen.toBuilder().status(CommandStatus.REVOKED).build();
				when(commandService.getById(anyString())).thenReturn(revokedCommand);

				var isRevoked = handler.isLoeschAnforderungZuruecknehmenRevoked(loeschAnforderungZuruecknehmen);

				assertThat(isRevoked).isTrue();
			}

			@Test
			void shouldReturnFalseOnOtherStatus() {
				var revokedCommand = loeschAnforderungZuruecknehmen.toBuilder().status(CommandStatus.FINISHED).build();
				when(commandService.getById(anyString())).thenReturn(revokedCommand);

				var isRevoked = handler.isLoeschAnforderungZuruecknehmenRevoked(loeschAnforderungZuruecknehmen);

				assertThat(isRevoked).isFalse();
			}
		}

		@Test
		void shouldReturnFalseOnNonMatchingOrder() {
			var isRevoked = handler.isLoeschAnforderungZuruecknehmenRevoked(CommandTestFactory.create());

			assertThat(isRevoked).isFalse();
		}
	}

	@DisplayName("translate order")
	@Nested
	class TestTranslateOrders {

		@ParameterizedTest
		@CsvSource(value = { "CREATE_KOMMENTAR;Kommentar", "CREATE_WIEDERVORLAGE;Wiedervorlage" }, delimiter = ';')
		void shouldTranslateToCreateOrder(String target, String itemName) {
			var command = handler.translateOrder(createCommand(itemName, CommandOrder.CREATE_ATTACHED_ITEM));

			assertThat(command.getOrder()).isEqualTo(target);
		}

		@ParameterizedTest
		@CsvSource(value = { "EDIT_KOMMENTAR;Kommentar", "EDIT_WIEDERVORLAGE;Wiedervorlage" }, delimiter = ';')
		void shouldTranslateToEditOrder(String target, String itemName) {
			var command = handler.translateOrder(createCommand(itemName, CommandOrder.UPDATE_ATTACHED_ITEM));

			assertThat(command.getOrder()).isEqualTo(target);
		}

		@ParameterizedTest
		@CsvSource(value = { "WIEDERVORLAGE_ERLEDIGEN;true", "WIEDERVORLAGE_WIEDEREROEFFNEN;false" }, delimiter = ';')
		void shouldTranslateToWiedervolageErledigenOrWiedervolageWiedereroeffnen(String target, String doneValue) {
			var command = handler.translateOrder(
					createCommand(Wiedervorlage.class.getSimpleName(), CommandOrder.PATCH_ATTACHED_ITEM, Boolean.valueOf(doneValue)));

			assertThat(command.getOrder()).isEqualTo(target);
		}

		@ParameterizedTest
		@CsvSource(value = { "RECEIVE_POSTFACH_NACHRICHT;IN" }, delimiter = ';')
		void shouldTranslateToReveivedIncomminNachricht(String target, String direction) {
			var command = handler.translateOrder(
					createCommand("OzgCloud_NachrichtenManager", CommandOrder.CREATE_ATTACHED_ITEM, direction));

			assertThat(command.getOrder()).isEqualTo(target);
		}

		@Test
		void shouldHandleMissingDone() {
			var command = handler.translateOrder(createCommand(Wiedervorlage.class.getSimpleName(), CommandOrder.PATCH_ATTACHED_ITEM));

			assertThat(command.getCommandOrder()).isEqualTo(CommandOrder.PATCH_ATTACHED_ITEM);
		}

		@Test
		void shouldHandleUnkownOrder() {
			var command = handler
					.translateOrder(createCommand(Wiedervorlage.class.getSimpleName(), CommandOrder.VORGANG_ABSCHLIESSEN));

			assertThat(command.getCommandOrder()).isEqualTo(CommandOrder.VORGANG_ABSCHLIESSEN);
		}

		@Test
		void shouldHandleMissingItemName() {
			var command = CommandTestFactory.createBuilder().order(LegacyOrder.EDIT_KOMMENTAR).body(Map.of("item", Map.of("value", "test")))
					.build();

			var translatedCommand = handler.translateOrder(command);

			assertThat(translatedCommand.getOrder()).isEqualTo(LegacyOrder.EDIT_KOMMENTAR);
		}

		private Command createCommand(String itemName, CommandOrder order) {
			return CommandTestFactory.createBuilder().order(order.name())
					.body(Map.of(VorgangAttachedItem.FIELD_ITEM_NAME, itemName, "item", Map.of("value", "test"))).build();
		}

		private Command createCommand(String itemName, CommandOrder order, boolean done) {
			return CommandTestFactory.createBuilder().order(order.name())
					.body(Map.of(VorgangAttachedItem.FIELD_ITEM_NAME, itemName, "item", Map.of("done", done)))
					.build();
		}

		private Command createCommand(String client, CommandOrder order, String direction) {
			return CommandTestFactory.createBuilder().order(order.name())
					.body(Map.of(HistorieCommandHandler.CLIENT, client, VorgangAttachedItem.FIELD_ITEM_NAME, PostfachMail.class.getSimpleName(),
							"item",
							Map.of(HistorieCommandHandler.DIRECTION, direction)))
					.build();
		}

	}

	@DisplayName("SEND_POSTFACH_MAIL")
	@Nested
	class TestSendPostfachMail {

		private Command command = CommandTestFactory.createBuilder().order(CommandOrder.SEND_POSTFACH_MAIL.name()).build();

		@Test
		void shouldHandleUnkownOrder() {
			var translatedCommand = handler.translateOrder(command);

			assertThat(translatedCommand.getCommandOrder()).isEqualTo(CommandOrder.SEND_POSTFACH_NACHRICHT);
		}
	}
}