/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.MockedStatic.Verification;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.BaseTypesMapper;
import de.ozgcloud.alfa.common.clientattribute.ClientAttribute;
import de.ozgcloud.alfa.common.clientattribute.ClientAttributeUtils;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserIdMapper;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttributeValue;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangWithEingang;

class VorgangWithEingangMapperTest {

	@Spy
	@InjectMocks
	private final VorgangWithEingangMapper mapper = Mappers.getMapper(VorgangWithEingangMapper.class);

	@Spy
	private final UserIdMapper userIdMapper = Mappers.getMapper(UserIdMapper.class);

	@Spy
	private final BaseTypesMapper baseTypesMapper = Mappers.getMapper(BaseTypesMapper.class);

	private MockedStatic<ClientAttributeUtils> utilsMockedStatic;

	@Mock
	private EingangMapper eingangMapper;

	@DisplayName("To VorgangWithEingang")
	@Nested
	class TestToVorgangWithEingang {
		private GrpcVorgangWithEingang grpcVorgangMitEingang = GrpcVorgangWithEingangTestFactory.create();
		private Verification staticClientAttributeUtilsCall = () -> ClientAttributeUtils.findGrpcValue(
				ClientAttribute.HAS_NEW_POSTFACH_NACHRICHT,
				grpcVorgangMitEingang.getClientAttributesList());

		@BeforeEach
		void setUpStaticMock() {
			utilsMockedStatic = mockStatic(ClientAttributeUtils.class);
		}

		@AfterEach
		void closeStaticMock() {
			utilsMockedStatic.close();
		}

		@Test
		void shouldMapCreatedAt() {
			var vorgang = callMapper();

			assertThat(vorgang.getCreatedAt()).isEqualTo(CREATED_AT);
		}

		@Test
		void shouldMapNummer() {
			var vorgang = callMapper();

			assertThat(vorgang.getNummer()).isEqualTo(NUMMER);
		}

		@Test
		void shouldMapAktenzeichen() {
			var vorgang = callMapper();

			assertThat(vorgang.getAktenzeichen()).isEqualTo(AKTENZEICHEN);
		}

		@Test
		void shouldMapStatus() {
			var vorgang = callMapper();

			assertThat(vorgang.getStatus()).isEqualTo(VorgangStatus.NEU);
		}

		@Test
		void shouldMapAssignedTo() {
			var vorgang = callMapper();

			assertThat(vorgang.getAssignedTo()).isEqualTo(UserId.from(ASSIGNED_TO));
		}

		@Test
		void shouldMapClientAttributeHasNewPostfachNachrichtToDefault() {
			utilsMockedStatic.when(staticClientAttributeUtilsCall)
					.thenReturn(Optional.empty());

			var vorgang = callMapper();

			assertThat(vorgang.isHasNewPostfachNachricht()).isFalse();
		}

		@ParameterizedTest
		@ValueSource(booleans = { true, false })
		void shouldMapClientAttributeHasNewPostfachNachrichtToAttributeValue(boolean value) {
			utilsMockedStatic
					.when(staticClientAttributeUtilsCall)
					.thenReturn(Optional.of(GrpcClientAttributeValue.newBuilder().setBoolValue(value).build()));

			var vorgang = callMapper();

			assertThat(vorgang.isHasNewPostfachNachricht()).isEqualTo(value);
		}

		private VorgangWithEingang callMapper() {
			return mapper.toVorgangWithEingang(grpcVorgangMitEingang);
		}
	}
}
